#include <cstdint>
#include <boost/format.hpp>
#include <string>

// Команды для компиляции:
// g++ -c --std=c++11 -fpic mylib.cpp -o mylib.o
// g++ -shared mylib.o -o libMyLib.so

// Макрос LIB_EXPORT используется для объявления функций, которые можно вызывать из динамической библиотеки.
// extern "C" при объявлении функции указывает, что для этой функции требуется использовать соглашение компановки
// для языка C. В таком случае генерируемое имя символа в таблице символов у итогового бинарного файла, полностью
// совпадает с именем функции. Например: для функции CalculateValue в таблице символов появится одноименный символ CalculateValue,
// который можно использовать для поиска адреса функции при подключении динамической библиотеки, например, с помощью dlsym(libHandle, "CalculateValue").
//
// __attribute__((visibility("default"))) указывает, что сгенерированный для функции символ будет иметь глобальную видимость в таблице символов,
// что позволяет искать его с помощью dlsym.
//
// __attribute__((sysv_abi)) указывает использование соглашение вызова функций sysv_abi, используемый по-умолчанию в семействе операционных систем
// GNU/Linux. Для архитектуры amd64 является единственно возможным вариантом. 
#if defined(_MSC_VER) || defined(WIN64) || defined(_WIN64) || defined(__WIN64__) || defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
    #if defined(__GNUC__)
        #define LIB_EXPORT extern "C" __attribute__((dllexport)) __attribute__((cdecl))
    #else
        #define LIB_EXPORT extern "C" __declspec(dllexport) __cdecl
    #endif
#else
    #define LIB_EXPORT extern "C" __attribute__((visibility("default"))) __attribute__((sysv_abi))
#endif

// При написании функций, вызываемых извне динамической библиотеки требуется использовать
// POD (Plain Old Data) типы, для обеспечения совместимости библиотеки с вызывающей программой.
// Для этого следует использовать целочисленные типы, объявленные в cstdint, так как стандартом
// гарантируется размер этих типов.
//

typedef _Float32 float32_t;
typedef _Float64 float64_t;

/// Расчет объема жидкости в вертикальном цилиндре гидростатическим методом
/// V = S * p / (ro * g)
/// Ускорение свободного падения м/с^2
constexpr double GRAVT_ACCEL = 9.80665;

LIB_EXPORT  float64_t  CalcVolumeInCylinder(float64_t cylinderArea, float64_t pressure, float64_t liquidDensity)  {
    return (cylinderArea * pressure) / (liquidDensity * GRAVT_ACCEL);
}

// Тип char имеет 8 бит в моделях данных LP32 ILP32 LLP64 LP64
// https://en.cppreference.com/w/cpp/language/types

/// Создание строки в соответствии с передаваемым форматом fmtstr, в который принимается единственный маркер
/// позиции "%1%" для числа double number. Результирующая строка записывается в result, имеющий максимальную
/// длину max_len байт.
LIB_EXPORT int8_t BoostFormatStringWithDouble(const char *fmtstr, float64_t number, char *result, uint32_t max_len)  {
    try {
        auto fmt = boost::format(fmtstr) % number;
        std::string tmp = fmt.str();
        if(tmp.size() > max_len)
            return -2;

        for(uint32_t i = 0; i < tmp.size(); i++) {
            result[i] = tmp[i];
        }

        return 0;
    }
    catch (...) {
        return -1;
    }
}

/// Возврат числа, умноженного на 2
LIB_EXPORT int8_t GetNumber(int8_t num)  {
    return 2*num;
}

